#!/usr/bin/env python2.4

"""
KodakLoader - tool for uploading photos to www.kodakgallery.com
Copyright (C) 2006 Jason D. Hildebrand 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Jason Hildebrand can be reached at <jason@opensky.ca>
"""


import pygtk
pygtk.require('2.0')

import gtk
import gobject
import uploader
import sys
import os
import ConfigParser

def scalePixBufToFit(pixbuf, avail_width, avail_height):
        actual_width, actual_height = pixbuf.get_width(), pixbuf.get_height()
        #print 'Actual image size: ', actual_width, actual_height
        #print 'Available space: ', avail_width, avail_height
        zoomratio = min(float(avail_width)/actual_width, float(avail_height)/actual_height, 1.0)
        return pixbuf.scale_simple(int(actual_width * zoomratio), int(actual_height * zoomratio), gtk.gdk.INTERP_BILINEAR)

DND_STRING = 0
dnd_types = [
    ('text/plain', 0, DND_STRING),
]

class MainWindow(gtk.Window):
    def __init__(self):
        gtk.Window.__init__(self)
        self.set_size_request(200, 400)

        self._filechooser = None

        # FIXME: should prompt user if an upload is in progress
        self.connect('destroy', lambda *w: gtk.main_quit())

        # create list view
        # create a TreeStore with one string column to use as the model
        self._liststore = gtk.ListStore(gtk.gdk.Pixbuf, str, str)
   
        self._treeview = gtk.TreeView(self._liststore)
        self._treeview.set_headers_visible(False)
        
        # create the TreeViewColumn to display the data
        self._tvcolumn1 = gtk.TreeViewColumn('Image')
        self._tvcolumn2 = gtk.TreeViewColumn('Filename')
        
        # add tvcolumn to treeview
        self._treeview.append_column(self._tvcolumn1)
        self._treeview.append_column(self._tvcolumn2)
        
        # create a CellRendererText to render the data
        self._cell1 = gtk.CellRendererPixbuf()
        self._cell2 = gtk.CellRendererText()
        
        # add the cell to the tvcolumn and allow it to expand
        self._tvcolumn1.pack_start(self._cell1, True)
        self._tvcolumn2.pack_start(self._cell2, True)
        
        # set the cell "text" attribute to column 0 - retrieve text
        # from that column in treestore
        self._tvcolumn1.set_attributes(self._cell1, pixbuf=0)
        self._tvcolumn2.set_attributes(self._cell2, text=1)
        
        # Allow sorting on the column
        self._tvcolumn2.set_sort_column_id(0)
        
        # Allow drag and drop reordering of rows
        self._treeview.set_reorderable(True)

        self._treeview.drag_dest_set(gtk.DEST_DEFAULT_ALL, dnd_types, gtk.gdk.ACTION_COPY | gtk.gdk.ACTION_MOVE)
        self._treeview.connect('drag_data_received', self.drag_data_received)
        self._treeview.connect('button-press-event', self.tree_view_button)
        
        scrolled = gtk.ScrolledWindow()
        scrolled.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolled.add_with_viewport(self._treeview)

        self._popup = popup = gtk.Menu()
        remove = gtk.MenuItem('Remove from list')
        remove.connect('activate', self.removeFiles)
        popup.append(remove)
        popup.show_all()

        self._upload = upload = gtk.Button('Upload')
        upload.set_sensitive(False)
        upload.connect('button-release-event', self.startUpload)
        self._select = select = gtk.Button('Add files...')
        select.connect('button-release-event', self.selectFiles)
        select.set_sensitive(False)
        self._statusbar = gtk.Statusbar()
        self._statusbarid = self._statusbar.get_context_id('main')
        self._progress_bar = progress_bar = gtk.ProgressBar()
        progress_bar.set_fraction(0)

        vbox = gtk.VBox()
        vbox.set_homogeneous(False)
        vbox.pack_start(select, expand=False, fill=False)
        vbox.pack_start(scrolled, fill=True)
        vbox.pack_start(upload, expand=False, fill=False)
        vbox.pack_start(progress_bar, expand=False, fill=False)
        vbox.pack_start(self._statusbar, expand=False, fill=False)
        #vbox.add(image)
        #vbox.add(progress_bar)
        self.add(vbox)

        self._browser = uploader.Browser()
        self.show_all()

    def ready(self, ready=True):
        if ready:
            self._upload.set_sensitive(len(self._liststore) > 0)
            self._select.set_sensitive(True)
        else:
            self._upload.set_sensitive(False)
            self._select.set_sensitive(False)

    def updateProgress(self, bytesTotal, bytesSent):
        #print bytesTotal, bytesSent
        self._progress_bar.set_fraction(float(bytesSent) / bytesTotal)
        self.updateDisplay()

    def updateDisplay(self):
        context = gobject.main_context_default()
        while 1:
            events = context.iteration(False)  # don't block
            if not events:
                break  # no events were processed

    def startUpload(self, one, two):
        self.ready(False)
        try:
            username, password = self.promptForCredentials()
            if username is None:
                return
            filenames = map(lambda row: row[2], iter(self._liststore))
            browser = self._browser
            self.startStatus('Connecting...')
            browser.login(username, password)
            self.endStatus()
            action, arg = self.promptForAlbum()
            self.startStatus('Uploading photos...')
            if action == 'existing':
                # add to existing album
                #print arg
                browser.uploadPhotos(filenames, collid=arg, callback=self.updateProgress)
            elif action == 'create':
                # create a new album
                browser.uploadPhotos(filenames, albumName=arg, callback=self.updateProgress)
            else:
                return
            self.updateProgress(1, 1)  # set to 100%
            self.endStatus()
            self.startStatus('Upload complete.')
            self._liststore.clear()
        finally:
            self.ready()

    def saveCredentials(self, username, password, savePassword):
        if not savePassword:
            password = ''
        config = ConfigParser.ConfigParser()
        config.read([os.path.expanduser('~/.kodakloader.conf')])
        if not config.has_section('credentials'):
            config.add_section('credentials')
        config.set('credentials', username, password)
        outfile = file(os.path.expanduser('~/.kodakloader.conf'), 'w')
        config.write(outfile)
        outfile.close()

    def storedCredentials(self):
        credentials = gtk.ListStore(str, str)
        config = ConfigParser.ConfigParser()
        config.read([os.path.expanduser('~/.kodakloader.conf')])
        if config.has_section('credentials'):
            for item in config.items('credentials'):
                return item
        return (None, None)

    def promptForCredentials(self):
        dialog = gtk.Dialog(title='Enter login information', parent=self, buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
        table = gtk.Table(rows=2, columns=2)
        username = gtk.Entry()
        password = gtk.Entry()
        savepassword = gtk.CheckButton('Save this password')
        password.set_visibility(False)
        u, p = self.storedCredentials()
        if u: username.set_text(u)
        if p: 
            password.set_text(p)
            savepassword.set_active(True)
        table.attach(gtk.Label('Email:'),    0, 1, 0, 1, 0, 0)
        table.attach(gtk.Label('Password:'), 0, 1, 1, 2, 0, 0)
        table.attach(username, 1, 2, 0, 1, 0, 0)
        table.attach(password, 1, 2, 1, 2, 0, 0)
        dialog.vbox.add(table)
        dialog.vbox.add(savepassword)
        dialog.vbox.show_all()
        response = dialog.run()
        try:
            if response == gtk.RESPONSE_ACCEPT:
                username, password = username.get_text(), password.get_text()
                self.saveCredentials(username, password, savepassword.get_active())
                return (username, password)
            else:
                return None, None
        finally:
            dialog.destroy()

    def promptForAlbum(self):
        self.startStatus('Retrieving list of albums...')
        albums = self._browser.albumList()
        self.endStatus()
        dialog = gtk.Dialog(title='Select destination', parent=self, buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
        dialog.set_response_sensitive(gtk.RESPONSE_ACCEPT, False)
        existing_album = gtk.RadioButton(None, 'Upload to existing album:')
        albumlist = gtk.combo_box_new_text()
        for album in albums:
            albumlist.append_text("%s (%s)" % (album[1], album[2]))
        albumlist.insert_text(0, "Select...")
        albumlist.set_active(0)
        hbox = gtk.HBox()
        hbox.add(existing_album)
        hbox.add(albumlist)
        dialog.vbox.pack_start(hbox, expand=False, fill=False)

        create_new_album = gtk.RadioButton(existing_album, 'Upload to new album:')
        newalbum = gtk.Entry()
        newalbum.set_sensitive(False)
        hbox = gtk.HBox()
        hbox.add(create_new_album)
        hbox.add(newalbum)

        def update_ok(widget):
            ok = (existing_album.get_active() and albumlist.get_active() > 0) \
              or (create_new_album.get_active() and newalbum.get_text() != '')
            dialog.set_response_sensitive(gtk.RESPONSE_ACCEPT, ok)

        def radio_changed(widget):
            existing = existing_album.get_active()
            albumlist.set_sensitive(existing)
            newalbum.set_sensitive(not existing)
            update_ok(widget)

        existing_album.connect('toggled', radio_changed)
        newalbum.connect('changed', update_ok)
        albumlist.connect('changed', update_ok)

        dialog.vbox.pack_start(hbox, expand=False, fill=False)
        dialog.vbox.show_all()
        response = dialog.run()
        try:
            if response == gtk.RESPONSE_ACCEPT:
                if existing_album.get_active():
                    return 'existing', albums[albumlist.get_active() - 1][0]
                return 'create', newalbum.get_text()
            else:
                return None, None
        finally:
            dialog.destroy()

    def startStatus(self, message):
        self._statusbar.push(self._statusbarid, message)
        self.updateDisplay()

    def endStatus(self):
        self._statusbar.pop(self._statusbarid)
        self.updateDisplay()

    def addFile(self, file):
        self.startStatus('Loading photos...')
        try:
            pixbuf = gtk.gdk.pixbuf_new_from_file(file)
        except gobject.GError:
            pass  # wasn't a file that we could handle
        else:
            pixbuf = scalePixBufToFit(pixbuf, 50, 50)
            self._liststore.append((pixbuf, os.path.basename(file), file))
        self.endStatus()

    def removeFiles(self, item):
        liststore, iter = self._treeview.get_selection().get_selected()
        liststore.remove(iter)
        self.ready()

    def updateChooserPreview(self, chooser, image):
        filename = chooser.get_preview_filename()
        if filename is None:
            image.set_from_pixbuf(self.render_icon(gtk.STOCK_DIALOG_QUESTION, gtk.ICON_SIZE_DIALOG))
        else:
            try:
                pixbuf = gtk.gdk.pixbuf_new_from_file(filename)
            except gobject.GError:
                image.set_from_pixbuf(self.render_icon(gtk.STOCK_DIALOG_QUESTION, gtk.ICON_SIZE_DIALOG))
            else:
                pixbuf = scalePixBufToFit(pixbuf, 150, 150)
                image.set_from_pixbuf(pixbuf)

    def selectFiles(self, one, two):
        if self._filechooser:
            chooser = self._filechooser
        else:
            chooser = self._filechooser = gtk.FileChooserDialog('Select files for upload', self, 0, (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
            chooser.set_local_only(True)
            chooser.set_select_multiple(True)
            image = gtk.Image()
            chooser.set_preview_widget(image)
            chooser.connect('selection-changed', self.updateChooserPreview, image)

        chooser.show_all()
        response = chooser.run()
        chooser.hide()
        if response == gtk.RESPONSE_ACCEPT:
            for file in chooser.get_filenames():
                self.addFile(file)
            self.ready()

    def drag_data_received(self, item, context, x, y, data, info, time):
        #print item, context, x, y, data, info, time
        if data.format == 8:
            files = data.data.strip()
            #print files
            #print files.split('\n')
            for filepath in files.split('\n'):
                filepath = filepath.strip()
                #print 'Received "%s"' % filepath
                if filepath.startswith('file://'):
                    filepath = filepath[7:]
                self.addFile(filepath)
            self.ready()
            context.finish(True, False, time)
        else:
            context.finish(False, False, time)

    def tree_view_button(self, widget, event):
        if event.button == 3:
            treeview = self._treeview
            path, col, cellx, celly = treeview.get_path_at_pos(
    int(event.x), int(event.y) )
            treeview.grab_focus()
            treeview.set_cursor( path, col, 0)
            self._popup.popup(None, None, None, 3, gtk.get_current_event_time())
            return 1

def run():
    m = MainWindow()
    m.set_title('Kodak Loader')
    for f in sys.argv[1:]:
        m.addFile(f)
    m.ready()
    gtk.main()
