#!/usr/bin/env python2.4

"""
KodakLoader - tool for uploading photos to www.kodakgallery.com
Copyright (C) 2006 Jason D. Hildebrand 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Jason Hildebrand can be reached at <jason@opensky.ca>
"""

import mechanize
import re
import urllib2
import httplib
import time
import sys
import os

filesPerPage = 2

uploadCallBack = None
uploadBytesSent = None
uploadBytesTotal = None

class HTTPConnection(httplib.HTTPConnection):
    def send(self, str):
        global uploadBytesTotal, uploadBytesSent
        uploadBytesSent = 0
        uploadBytesTotal = send_length = len(str)
        chunksize = 1024
        while uploadBytesSent < send_length:
            if uploadCallBack:
                uploadCallBack(uploadBytesTotal, uploadBytesSent) 
            sys.stdout.flush()
            t1 = time.time() 
            httplib.HTTPConnection.send(self, str[uploadBytesSent:uploadBytesSent+chunksize])
            uploadBytesSent += chunksize
            t2 = time.time() 

            # adjust chunksize so that it takes between .5 and 2 
            # seconds to send a chunk
            if t2 - t1 < .5:
                chunksize *= 2
            elif t2 - t1 > 2:
                chunksize /= 2

class HTTPHandler(urllib2.AbstractHTTPHandler):

    def http_open(self, req):
        return self.do_open(HTTPConnection, req)

    http_request = urllib2.AbstractHTTPHandler.do_request_

class Browser(mechanize.Browser):
    handler_classes = mechanize.UserAgent.handler_classes
    handler_classes['http'] = HTTPHandler

    def __init__(self, *args, **kwargs):
        mechanize.Browser.__init__(self, *args, **kwargs)
        self.set_debug_http(False)
        self.set_handle_robots(False)
        self._logged_in = False

    def printPage(self):
        for line in self.response().readlines():
            print line

    def login(self, username, password):
        """
        Log in if necessary.
        """
        if self._logged_in:
            self.open("http://www.kodakgallery.com/MyKodak.jps")
            self._logged_in = self.response().read().find('Account Information') != -1

        if not self._logged_in:
            self.open("http://www.kodakgallery.com/Signin.jsp")

            self.select_form(nr=0)
            self["email"] = username
            self["password"] = password
            response = self.submit()
            self._logged_in = response.read().find('Account Information') != -1
            assert self._logged_in

    def albumsInPage(self):
        '''
        The purpose of this method is to locate the album links (collids), names and dates
        in HTML like this:

        <td class="albumcell">
<span class="albumlink-span"><a class="albumlink" href="BrowsePhotos.jsp?&amp;collid=45067316309.37442283709.1142571080194&amp;page=1&amp;sort_order=0&amp;albumsperpage=12&amp;navfolderid=0&amp;ownerid=0" >
<img class="shim" src="images/common/albumshim.gif" alt="" /><img width="72" height="96" src="http://images.kodakgallery.com/photos1867/7/83/22/44/27/0/27442283709_0_SM.jpg" alt="" name="i_37442283709_main" /></a></span>
<p class="albumname"><a href="BrowsePhotos.jsp?&amp;collid=45067316309.37442283709.1142571080194&amp;page=1&amp;sort_order=0&amp;albumsperpage=12&amp;navfolderid=0&amp;ownerid=0">test</a></p>
<p>
3/15/06
(1 photo)</p>
</td>
        '''
        response = self.response().read()
        return re.findall('(?ms)<a[^>]*href="BrowsePhotos[^\"]*collid=([0-9\.]*)[^\"]*"[^>]*>([^<]*)</a>\s*</p>\s*<p>\s*(\S*)\s*\(.*?</p>', response)

    def albumList(self):
        """
        Return a list of albums.  Each album is represented by a tuple
        consisting of (collid, name, date).
        """
        self.follow_link(text_regex=re.compile('View.*Edit Photos'))
        self.follow_link(text_regex=re.compile("All My Albums.*"))
        albumlinks0 = self.links(url_regex=re.compile("BrowsePhotos.*"))
        albumlinks = self.albumsInPage()
        pages = self.links(url_regex=re.compile("AlbumMenu.*[\?\&]page=.*"))
        gotpage = {}
        for page in pages:
            #print page
            pagenum = re.search('[\?\&]page=(\d*)', page.url)
            if pagenum:
                pagenum = int(pagenum.group(1))
            else:
                pagenum = 0
            if gotpage.has_key(pagenum):
                continue
            gotpage[pagenum] = 1
            self.follow_link(page)
            #print page.url
            #albumlinks += self.links(url_regex=re.compile("BrowsePhotos.*"))
            albumlinks += self.albumsInPage()
        #return filter(lambda link: link.find('[IMG]') == -1, map(lambda link: link.text, albumlinks))
        return albumlinks

    def uploadPhotos(self, files, collid=None, albumName='', callback=None, totalSize=0, totalSent=0):
        global uploadCallBack

        if collid:
            self.open('http://www.kodakgallery.com/CopyPictures.jsp?mode=add&dest_collid=' + collid)
            self.select_form(predicate=lambda f: f.attrs['id'] == 'uploadsniff_form')
            self.form.set_all_readonly(False)
            self['browserCode'] = ''
            self['pluginInstalled'] = ''
            self['collid'] = collid
            self['upload'] = 'true'
            self.submit()
        else:
            # create a new album
            response = self.follow_link(text="Upload Photos")
            self.select_form(name='ofoto_uploadform')
            self["name"] = albumName
            self.submit()

        if totalSize == 0:
            for f in files:
                totalSize += os.stat(f)[6]
            #print 'totalsize: %d' % totalSize

        self.select_form(name='ofoto_uploadBrowseform2')

        # calculate size of this submission and populate the web form
        i = 0
        submitsize = 0
        for f in files[:filesPerPage]:
            submitsize += os.stat(f)[6]
            self.add_file(file(f), 'image/jpeg', os.path.basename(f), name='image_file_%d' % (i + 1))
            i += 1

        # wrap the callback, so that we can report the total bytes and sent bytes
        # for the entire upload (which could be split over multiple transactions)
        def updateDisplay(submittotal, submitsent):
            if submittotal > submitsize:  # we only care about main http POST, not related GETs.
                callback(totalSize, totalSent + submitsent)

        uploadCallBack = updateDisplay
        try:
            self.submit()

            # if necessary, pick out the collid and upload remaining files to the same album.
            if len(files) > filesPerPage:
                collid = re.search("<a[^>]*setValues\('([0-9\.]*)[^>]*>Upload more", self.response().read()).group(1)
                self.uploadPhotos(files[filesPerPage:], collid=collid, callback=callback, totalSize=totalSize, totalSent=(totalSent + submitsize))
        finally:
            uploadCallBack = None


# for testing only
if __name__ == '__main__':
    b = Browser()
    b.login(sys.argv[1], sys.argv[2])
    b.uploadPhotos(sys.argv[3:], '45067316309.89566433709.1142703711860')
